@@ -50,10 +50,6 @@
 #include "udiskslinuxlogicalvolumeobject.h"
 
 #include "udiskslvm2daemonutil.h"
-#include "udiskslvm2dbusutil.h"
-#include "udiskslvm2util.h"
-#include "udisks-lvm2-generated.h"
-
 #include "jobhelpers.h"
 
 /**
@@ -140,11 +136,14 @@ udisks_linux_volume_group_update (UDisksLinuxVolumeGroup *volume_group,
                                   gboolean               *needs_polling_ret)
 {
   UDisksVolumeGroup *iface = UDISKS_VOLUME_GROUP (volume_group);
+
   udisks_volume_group_set_name (iface, vg_info->name);
   udisks_volume_group_set_uuid (iface, vg_info->uuid);
   udisks_volume_group_set_size (iface, vg_info->size);
   udisks_volume_group_set_free_size (iface, vg_info->free);
   udisks_volume_group_set_extent_size (iface, vg_info->extent_size);
+
+  g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (iface));
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -192,7 +191,7 @@ udisks_linux_volume_group_get_logical_volumes (UDisksVolumeGroup *group,
   objects = udisks_daemon_get_objects (daemon);
   for (l = objects; l != NULL; l = l->next)
     {
-      volume = udisks_object_peek_logical_volume (UDISKS_OBJECT(l->data));
+      volume = udisks_object_peek_logical_volume (UDISKS_OBJECT (l->data));
       if (volume &&
           g_strcmp0 (udisks_logical_volume_get_volume_group (volume),
                      g_dbus_object_get_object_path (object)) == 0)
@@ -248,6 +247,7 @@ handle_delete (UDisksVolumeGroup     *_group,
   GError *error = NULL;
   UDisksLinuxVolumeGroup *group = UDISKS_LINUX_VOLUME_GROUP (_group);
   UDisksLinuxVolumeGroupObject *object = NULL;
+  UDisksLinuxModuleLVM2 *module;
   UDisksDaemon *daemon;
   uid_t caller_uid;
   gboolean teardown_flag = FALSE;
@@ -264,7 +264,8 @@ handle_delete (UDisksVolumeGroup     *_group,
       goto out;
     }
 
-  daemon = udisks_linux_volume_group_object_get_daemon (object);
+  module = udisks_linux_volume_group_object_get_module (object);
+  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
 
   /* Find physical volumes to wipe. */
   if (arg_wipe)
@@ -296,7 +297,7 @@ handle_delete (UDisksVolumeGroup     *_group,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
                                      UDISKS_OBJECT (object),
-                                     lvm2_policy_action_id,
+                                     LVM2_POLICY_ACTION_ID,
                                      arg_options,
                                      N_("Authentication is required to delete a volume group"),
                                      invocation);
@@ -350,14 +351,20 @@ handle_delete (UDisksVolumeGroup     *_group,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+typedef struct
+{
+  UDisksLinuxModuleLVM2 *module;
+  const gchar *name;
+} WaitForVolumeGroupObjectData;
+
 static UDisksObject *
 wait_for_volume_group_object (UDisksDaemon *daemon,
                               gpointer      user_data)
 {
-  const gchar *name = user_data;
+  WaitForVolumeGroupObjectData *data = user_data;
   UDisksLinuxVolumeGroupObject *object;
 
-  object = udisks_daemon_util_lvm2_find_volume_group_object (daemon, name);
+  object = udisks_linux_module_lvm2_find_volume_group_object (data->module, data->name);
 
   if (object == NULL)
     return NULL;
@@ -374,10 +381,12 @@ handle_rename (UDisksVolumeGroup     *_group,
   GError *error = NULL;
   UDisksLinuxVolumeGroup *group = UDISKS_LINUX_VOLUME_GROUP (_group);
   UDisksLinuxVolumeGroupObject *object = NULL;
+  UDisksLinuxModuleLVM2 *module;
   UDisksDaemon *daemon;
   uid_t caller_uid;
   UDisksObject *group_object = NULL;
   VGJobData data;
+  WaitForVolumeGroupObjectData wait_data;
 
   object = udisks_daemon_util_dup_object (group, &error);
   if (object == NULL)
@@ -386,7 +395,8 @@ handle_rename (UDisksVolumeGroup     *_group,
       goto out;
     }
 
-  daemon = udisks_linux_volume_group_object_get_daemon (object);
+  module = udisks_linux_volume_group_object_get_module (object);
+  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
 
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
@@ -402,7 +412,7 @@ handle_rename (UDisksVolumeGroup     *_group,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
                                      UDISKS_OBJECT (object),
-                                     lvm2_policy_action_id,
+                                     LVM2_POLICY_ACTION_ID,
                                      options,
                                      N_("Authentication is required to rename a volume group"),
                                      invocation);
@@ -429,11 +439,13 @@ handle_rename (UDisksVolumeGroup     *_group,
       goto out;
     }
 
+  wait_data.module = module;
+  wait_data.name = new_name;
   group_object = udisks_daemon_wait_for_object_sync (daemon,
                                                      wait_for_volume_group_object,
-                                                     (gpointer) new_name,
+                                                     &wait_data,
                                                      NULL,
-                                                     10, /* timeout_seconds */
+                                                     UDISKS_DEFAULT_WAIT_TIMEOUT,
                                                      &error);
   if (group_object == NULL)
     {
@@ -463,6 +475,7 @@ handle_add_device (UDisksVolumeGroup     *_group,
                    GVariant              *options)
 {
   UDisksLinuxVolumeGroup *group = UDISKS_LINUX_VOLUME_GROUP (_group);
+  UDisksLinuxModuleLVM2 *module;
   UDisksDaemon *daemon;
   UDisksLinuxVolumeGroupObject *object;
   uid_t caller_uid;
@@ -479,7 +492,8 @@ handle_add_device (UDisksVolumeGroup     *_group,
       goto out;
     }
 
-  daemon = udisks_linux_volume_group_object_get_daemon (object);
+  module = udisks_linux_volume_group_object_get_module (object);
+  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
 
   error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
@@ -512,7 +526,7 @@ handle_add_device (UDisksVolumeGroup     *_group,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
                                      UDISKS_OBJECT (object),
-                                     lvm2_policy_action_id,
+                                     LVM2_POLICY_ACTION_ID,
                                      options,
                                      N_("Authentication is required to add a device to a volume group"),
                                      invocation);
@@ -598,6 +612,7 @@ handle_remove_common (UDisksVolumeGroup     *_group,
                       gboolean               arg_wipe)
 {
   UDisksLinuxVolumeGroup *group = UDISKS_LINUX_VOLUME_GROUP (_group);
+  UDisksLinuxModuleLVM2 *module;
   UDisksDaemon *daemon;
   UDisksLinuxVolumeGroupObject *object;
   uid_t caller_uid;
@@ -633,7 +648,8 @@ handle_remove_common (UDisksVolumeGroup     *_group,
       goto out;
     }
 
-  daemon = udisks_linux_volume_group_object_get_daemon (object);
+  module = udisks_linux_volume_group_object_get_module (object);
+  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
 
   error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
@@ -666,7 +682,7 @@ handle_remove_common (UDisksVolumeGroup     *_group,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
                                      UDISKS_OBJECT (object),
-                                     lvm2_policy_action_id,
+                                     LVM2_POLICY_ACTION_ID,
                                      options,
                                      authentication_error_msg,
                                      invocation);
@@ -780,18 +796,20 @@ wait_for_logical_volume_path (UDisksLinuxVolumeGroupObject  *group_object,
                               GError                       **error)
 {
   struct WaitData data;
+  UDisksLinuxModuleLVM2 *module;
   UDisksDaemon *daemon;
   UDisksObject *volume_object;
   const gchar *object_path;
 
   data.group_object = group_object;
   data.name = name;
-  daemon = udisks_linux_volume_group_object_get_daemon (group_object);
+  module = udisks_linux_volume_group_object_get_module (group_object);
+  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
   volume_object = udisks_daemon_wait_for_object_sync (daemon,
                                                       wait_for_logical_volume_object,
                                                       &data,
                                                       NULL,
-                                                      10, /* timeout_seconds */
+                                                      UDISKS_DEFAULT_WAIT_TIMEOUT,
                                                       error);
   if (volume_object == NULL)
     return NULL;
@@ -804,7 +822,7 @@ wait_for_logical_volume_path (UDisksLinuxVolumeGroupObject  *group_object,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-enum VolumeType { VOL_PLAIN, VOL_THIN_POOL, VOL_THIN_VOLUME };
+enum VolumeType { VOL_PLAIN, VOL_THIN_POOL, VOL_THIN_VOLUME, VOL_VDO_VOLUME };
 typedef void (*VolumeCompletionFunc) (UDisksVolumeGroup     *object,
                                       GDBusMethodInvocation *invocation,
                                       const gchar           *result);
@@ -816,15 +834,21 @@ handle_create_volume (UDisksVolumeGroup              *_group,
                       guint64                         arg_size,
                       GVariant                       *options,
                       enum VolumeType                 vol_creation_type,
-                      const gchar                    *arg_pool)
+                      const gchar                    *arg_pool,
+                      guint64                         arg_virtual_size,
+                      guint64                         arg_index_memory,
+                      gboolean                        arg_compression,
+                      gboolean                        arg_deduplication,
+                      const gchar                    *arg_write_policy)
 {
   GError *error = NULL;
   UDisksLinuxVolumeGroup *group = UDISKS_LINUX_VOLUME_GROUP (_group);
   UDisksLinuxVolumeGroupObject *object = NULL;
+  UDisksLinuxModuleLVM2 *module;
   UDisksDaemon *daemon;
   uid_t caller_uid;
   const gchar *lv_objpath;
-  LVJobData data;
+  LVJobData data = {0};
   UDisksLinuxLogicalVolumeObject *pool_object = NULL;
   const gchar *auth_error_msg = NULL;
   UDisksThreadedJobFunc create_function = NULL;
@@ -842,12 +866,18 @@ handle_create_volume (UDisksVolumeGroup              *_group,
       create_function = lvcreate_thin_job_func;
       completion_function = udisks_volume_group_complete_create_thin_volume;
     }
-  else
+  else if (VOL_THIN_POOL == vol_creation_type)
     {
       auth_error_msg = N_("Authentication is required to create a thin pool volume");
       create_function = lvcreate_thin_pool_job_func;
       completion_function = udisks_volume_group_complete_create_thin_pool_volume;
     }
+  else if (VOL_VDO_VOLUME == vol_creation_type)
+    {
+      auth_error_msg = N_("Authentication is required to create a VDO volume");
+      create_function = lvcreate_vdo_job_func;
+      completion_function = udisks_volume_group_complete_create_vdo_volume;
+    }
 
   object = udisks_daemon_util_dup_object (group, &error);
   if (object == NULL)
@@ -856,8 +886,8 @@ handle_create_volume (UDisksVolumeGroup              *_group,
       goto out;
     }
 
-  daemon = udisks_linux_volume_group_object_get_daemon (object);
-
+  module = udisks_linux_volume_group_object_get_module (object);
+  daemon = udisks_module_get_daemon (UDISKS_MODULE (module));
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
                                                NULL /* GCancellable */,
@@ -872,7 +902,7 @@ handle_create_volume (UDisksVolumeGroup              *_group,
   /* Policy check. */
   UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
                                      UDISKS_OBJECT (object),
-                                     lvm2_policy_action_id,
+                                     LVM2_POLICY_ACTION_ID,
                                      options,
                                      auth_error_msg,
                                      invocation);
@@ -896,6 +926,16 @@ handle_create_volume (UDisksVolumeGroup              *_group,
       data.pool_name = udisks_linux_logical_volume_object_get_name (pool_object);
     }
 
+  if (VOL_VDO_VOLUME == vol_creation_type)
+    {
+      data.pool_name = arg_pool;
+      data.virtual_size = arg_virtual_size;
+      data.index_memory = arg_index_memory;
+      data.compression = arg_compression;
+      data.deduplication = arg_deduplication;
+      data.write_policy = arg_write_policy;
+    }
+
   if (!udisks_daemon_launch_threaded_job_sync (daemon,
                                                UDISKS_OBJECT (object),
                                                "lvm-vg-create-volume",
@@ -941,7 +981,7 @@ handle_create_plain_volume (UDisksVolumeGroup     *_group,
                             GVariant              *options)
 {
   return handle_create_volume(_group, invocation, arg_name, arg_size, options,
-                              VOL_PLAIN, NULL);
+                              VOL_PLAIN, NULL, 0, 0, FALSE, FALSE, NULL);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -954,7 +994,7 @@ handle_create_thin_pool_volume (UDisksVolumeGroup     *_group,
                                 GVariant              *options)
 {
   return handle_create_volume(_group, invocation, arg_name, arg_size, options,
-                              VOL_THIN_POOL, NULL);
+                              VOL_THIN_POOL, NULL, 0, 0, FALSE, FALSE, NULL);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -968,7 +1008,25 @@ handle_create_thin_volume (UDisksVolumeGroup     *_group,
                            GVariant              *options)
 {
   return handle_create_volume(_group, invocation, arg_name, arg_size, options,
-                              VOL_THIN_VOLUME, arg_pool);
+                              VOL_THIN_VOLUME, arg_pool, 0, 0, FALSE, FALSE, NULL);
+}
+
+static gboolean
+handle_create_vdo_volume (UDisksVolumeGroup     *_group,
+                          GDBusMethodInvocation *invocation,
+                          const gchar           *arg_name,
+                          const gchar           *arg_pool,
+                          guint64                arg_size,
+                          guint64                arg_virtual_size,
+                          guint64                arg_index_memory,
+                          gboolean               arg_compression,
+                          gboolean               arg_deduplication,
+                          const gchar           *arg_write_policy,
+                          GVariant              *options)
+{
+  return handle_create_volume (_group, invocation, arg_name, arg_size, options,
+                               VOL_VDO_VOLUME, arg_pool, arg_virtual_size, arg_index_memory,
+                               arg_compression, arg_deduplication, arg_write_policy);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -988,4 +1046,6 @@ volume_group_iface_init (UDisksVolumeGroupIface *iface)
   iface->handle_create_plain_volume = handle_create_plain_volume;
   iface->handle_create_thin_pool_volume = handle_create_thin_pool_volume;
   iface->handle_create_thin_volume = handle_create_thin_volume;
+
+  iface->handle_create_vdo_volume = handle_create_vdo_volume;
 }
