@@ -2,11 +2,17 @@ import dbus
 import os
 import re
 import six
+import shutil
+import tarfile
 import tempfile
 import time
 import unittest
 import configparser
 
+import gi
+gi.require_version('BlockDev', '2.0')
+from gi.repository import BlockDev
+
 from distutils.version import LooseVersion
 
 import udiskstestcase
@@ -15,16 +21,25 @@ import udiskstestcase
 UDISKS_CONFIG_FILE = "/etc/udisks2/udisks2.conf"
 
 
+def _get_cryptsetup_version():
+    _ret, out = udiskstestcase.UdisksTestCase.run_command('cryptsetup --version')
+    m = re.search(r'cryptsetup ([\d\.]+)', out)
+    if not m or len(m.groups()) != 1:
+        raise RuntimeError('Failed to determine cryptsetup version from: %s' % out)
+    return LooseVersion(m.groups()[0])
+
+
+def _get_blkid_version():
+    _ret, out = udiskstestcase.UdisksTestCase.run_command('blkid --version')
+    m = re.search(r'blkid from util-linux ([\d\.]+)', out)
+    if not m or len(m.groups()) != 1:
+        raise RuntimeError('Failed to determine blkid version from: %s' % out)
+    return LooseVersion(m.groups()[0])
+
+
 class UdisksEncryptedTest(udiskstestcase.UdisksTestCase):
     '''This is an encrypted device test suite'''
 
-    def _get_cryptsetup_version(self):
-        _ret, out = self.run_command('cryptsetup --version')
-        m = re.search(r'cryptsetup ([\d\.]+)', out)
-        if not m or len(m.groups()) != 1:
-            raise RuntimeError('Failed to determine cryptsetup version from: %s' % out)
-        return LooseVersion(m.groups()[0])
-
     def _create_luks(self, device, passphrase, binary=False):
         raise NotImplementedError()
 
@@ -41,9 +56,6 @@ class UdisksEncryptedTest(udiskstestcase.UdisksTestCase):
         d['erase'] = True
         device.Format('empty', d, dbus_interface=self.iface_prefix + '.Block')
 
-    def _unmount(self, disk_path):
-        self.run_command('umount %s' % disk_path)
-
     def test_create(self):
         disk_name = os.path.basename(self.vdevs[0])
         disk = self.get_object('/block_devices/' + disk_name)
@@ -160,7 +172,25 @@ class UdisksEncryptedTest(udiskstestcase.UdisksTestCase):
         dbus_cleartext = self.get_property(disk, '.Encrypted', 'CleartextDevice')
         dbus_cleartext.assertEqual(luks)
 
-    @unittest.skipUnless("JENKINS_HOME" in os.environ, "skipping test that modifies system configuration")
+        disk.Lock(self.no_options, dbus_interface=self.iface_prefix + '.Encrypted')
+        self.assertFalse(os.path.exists('/dev/disk/by-uuid/%s' % luks_uuid))
+
+        dbus_cleartext = self.get_property(disk, '.Encrypted', 'CleartextDevice')
+        dbus_cleartext.assertEqual('/')
+
+        # read-only
+        ro_opts = dbus.Dictionary({'read-only': dbus.Boolean(True)}, signature=dbus.Signature('sv'))
+        luks = disk.Unlock('test', ro_opts,
+                           dbus_interface=self.iface_prefix + '.Encrypted')
+        self.assertIsNotNone(luks)
+        self.assertTrue(os.path.exists('/dev/disk/by-uuid/%s' % luks_uuid))
+
+        luks_obj = self.get_object(luks)
+        self.assertIsNotNone(luks_obj)
+        luks_ro = self.get_property(luks_obj, '.Block', 'ReadOnly')
+        luks_ro.assertTrue()
+
+    @udiskstestcase.tag_test(udiskstestcase.TestTags.UNSAFE)
     def test_open_crypttab(self):
         # this test will change /etc/crypttab, we might want to revert the changes when it finishes
         crypttab = self.read_file('/etc/crypttab')
@@ -185,8 +215,6 @@ class UdisksEncryptedTest(udiskstestcase.UdisksTestCase):
         new_crypttab = crypttab + '%s UUID=%s none\n' % (luks_name, disk_uuid)
         self.write_file('/etc/crypttab', new_crypttab)
 
-        # give udisks time to react to change of the file
-        time.sleep(5)
         dbus_conf = disk.GetSecretConfiguration(self.no_options, dbus_interface=self.iface_prefix + '.Block')
         self.assertIsNotNone(dbus_conf)
         self.assertEqual(self.ay_to_str(dbus_conf[0][1]['name']), luks_name)
@@ -206,7 +234,7 @@ class UdisksEncryptedTest(udiskstestcase.UdisksTestCase):
         luks_path = self.get_property(luks_obj, '.Block', 'PreferredDevice')
         luks_path.assertEqual(self.str_to_ay(dm_path))
 
-    @unittest.skipUnless("JENKINS_HOME" in os.environ, "skipping test that modifies system configuration")
+    @udiskstestcase.tag_test(udiskstestcase.TestTags.UNSAFE)
     def test_open_crypttab_keyfile(self):
         # this test will change /etc/crypttab, we might want to revert the changes when it finishes
         crypttab = self.read_file('/etc/crypttab')
@@ -237,8 +265,6 @@ class UdisksEncryptedTest(udiskstestcase.UdisksTestCase):
         new_crypttab = crypttab + '%s UUID=%s %s\n' % (luks_name, disk_uuid, key_file)
         self.write_file('/etc/crypttab', new_crypttab)
 
-        # give udisks time to react to change of the file
-        time.sleep(5)
         dbus_conf = disk.GetSecretConfiguration(self.no_options, dbus_interface=self.iface_prefix + '.Block')
         self.assertIsNotNone(dbus_conf)
         self.assertEqual(self.ay_to_str(dbus_conf[0][1]['name']), luks_name)
@@ -275,7 +301,7 @@ class UdisksEncryptedTest(udiskstestcase.UdisksTestCase):
 
         mnt_path = luks.Mount(self.no_options, dbus_interface=self.iface_prefix + '.Filesystem')
         self.assertIsNotNone(mnt_path)
-        self.addCleanup(self._unmount, mnt_path)
+        self.addCleanup(self.try_unmount, mnt_path)
 
         # should not be possible to close mounted luks
         msg = 'org.freedesktop.UDisks2.Error.Failed: Error locking'
@@ -337,6 +363,70 @@ class UdisksEncryptedTest(udiskstestcase.UdisksTestCase):
         clear_size3 = self.get_block_size(clear_dev)
         self.assertEqual(clear_size3, clear_size)
 
+    @udiskstestcase.tag_test(udiskstestcase.TestTags.UNSAFE)
+    def test_configuration_track_parents(self):
+        # this test will change /etc/crypttab and /etc/fstab, we might want to revert the changes
+        # in case something goes wrong
+        crypttab = self.read_file('/etc/crypttab')
+        self.addCleanup(self.write_file, '/etc/crypttab', crypttab)
+        fstab = self.read_file('/etc/fstab')
+        self.addCleanup(self.write_file, '/etc/fstab', fstab)
+
+        passphrase = 'test'
+
+        disk_name = os.path.basename(self.vdevs[0])
+        disk = self.get_object('/block_devices/' + disk_name)
+
+        disk.Format('dos', self.no_options, dbus_interface=self.iface_prefix + '.Block')
+        path = disk.CreatePartition(dbus.UInt64(1024**2), dbus.UInt64(100 * 1024**2), '', '',
+                                    self.no_options, dbus_interface=self.iface_prefix + '.PartitionTable')
+        part = self.bus.get_object(self.iface_prefix, path)
+        self.assertIsNotNone(part)
+
+        # create LUKS on the partition and add it to crypttab
+        self._create_luks(part, passphrase)
+        self.udev_settle()
+
+        conf = dbus.Dictionary({'name': self.str_to_ay('udisks_luks_test'),
+                                'device': self.str_to_ay(self.vdevs[0] + '1'),
+                                'passphrase-contents': self.str_to_ay(''),
+                                'options': self.str_to_ay('')},
+                               signature=dbus.Signature('sv'))
+        part.AddConfigurationItem(('crypttab', conf), self.no_options,
+                                  dbus_interface=self.iface_prefix + '.Block')
+
+        luks_path = self.get_property(part, '.Encrypted', 'CleartextDevice')
+        luks_dev = self.bus.get_object(self.iface_prefix, luks_path.value)
+        self.assertIsNotNone(luks_dev)
+
+        # format the LUKS device to ext4 and add it to fstab
+        conf_items = dbus.Dictionary({'dir': self.str_to_ay('fakemountpoint'),
+                                      'type': self.str_to_ay('ext4'),
+                                      'opts': self.str_to_ay('defaults'),
+                                      'freq': 0, 'passno': 0,
+                                      'track-parents': True},
+                                     signature=dbus.Signature('sv'))
+        options = dbus.Dictionary({'config-items': dbus.Array([('fstab', conf_items)])}, signature=dbus.Signature('sv'))
+        luks_dev.Format('ext4', options, dbus_interface=self.iface_prefix + '.Block')
+
+        luks_uuid = self.get_property_raw(luks_dev, '.Block', 'IdUUID')
+
+        # fstab configuration should be written to the ChildConfiguration of the partition device
+        child_conf = self.get_property(part, '.Encrypted', 'ChildConfiguration')
+        child_conf.assertTrue()
+        self.assertEqual(child_conf.value[0][0], 'fstab')
+
+        # wipe the LUKS partition
+        # with tear-down this should close the LUKS and remove enries from fstab and crypttab
+        options = dbus.Dictionary({'tear-down': True}, signature=dbus.Signature('sv'))
+        part.Format('empty', options, dbus_interface=self.iface_prefix + '.Block')
+
+        crypttab = self.read_file('/etc/crypttab')
+        self.assertNotIn('udisks_luks_test', crypttab)
+
+        fstab = self.read_file('/etc/fstab')
+        self.assertNotIn(luks_uuid, fstab)
+
 
 class UdisksEncryptedTestLUKS1(UdisksEncryptedTest):
     '''This is a LUKS1 encrypted device test suite'''
@@ -369,6 +459,16 @@ class UdisksEncryptedTestLUKS2(UdisksEncryptedTest):
     '''This is a LUKS2 encrypted device test suite'''
 
     luks_version = '2'
+    requested_plugins = BlockDev.plugin_specs_from_names(("crypto",))
+
+    @classmethod
+    def setUpClass(cls):
+        if not BlockDev.is_initialized():
+            BlockDev.init(cls.requested_plugins, None)
+        else:
+            BlockDev.reinit(cls.requested_plugins, True, None)
+
+        super(UdisksEncryptedTestLUKS2, cls).setUpClass()
 
     def _create_luks(self, device, passphrase, binary=False):
         options = dbus.Dictionary(signature='sv')
@@ -381,6 +481,17 @@ class UdisksEncryptedTestLUKS2(UdisksEncryptedTest):
         device.Format('xfs', options,
                       dbus_interface=self.iface_prefix + '.Block')
 
+    def _create_luks_integrity(self, device, passphrase):
+        if not BlockDev.utils_have_kernel_module('dm-integrity'):
+            self.skipTest('dm-integrity kernel module not available, skipping.')
+
+        # UDisks doesn't support creating LUKS2 with integrity, we need to use libblockdev
+        extra = BlockDev.CryptoLUKSExtra()
+        extra.integrity = 'hmac(sha256)'
+
+        BlockDev.crypto_luks_format(device, 'aes-cbc-essiv:sha256', 512, passphrase,
+                                    None, 0, BlockDev.CryptoLUKSVersion.LUKS2, extra)
+
     def _get_metadata_size_from_dump(self, disk):
         ret, out = self.run_command("cryptsetup luksDump %s" % disk)
         if ret != 0:
@@ -403,7 +514,7 @@ class UdisksEncryptedTestLUKS2(UdisksEncryptedTest):
         return m.group(1)
 
     def setUp(self):
-        cryptsetup_version = self._get_cryptsetup_version()
+        cryptsetup_version = _get_cryptsetup_version()
         if cryptsetup_version < LooseVersion('2.0.0'):
             self.skipTest('LUKS2 not supported')
 
@@ -423,7 +534,7 @@ class UdisksEncryptedTestLUKS2(UdisksEncryptedTest):
 
         # kernel keyring support and no passphrase for LUKS 2 given = fail
         if self._get_key_location('/dev/' + clear_dev) == 'keyring':
-            msg = 'org.freedesktop.UDisks2.Error.Failed: Error resizing encrypted device /dev/dm-[0-9]+: Insufficient persmissions to resize device. *'
+            msg = 'org.freedesktop.UDisks2.Error.Failed: Error resizing encrypted device /dev/dm-[0-9]+: Insufficient (permissions|persmissions) to resize device. *'
             with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
                 device.Resize(dbus.UInt64(100*1024*1024), self.no_options,
                               dbus_interface=self.iface_prefix + '.Encrypted')
@@ -500,4 +611,118 @@ class UdisksEncryptedTestLUKS2(UdisksEncryptedTest):
         default_encryption_type = self.get_property(manager, '.Manager', 'DefaultEncryptionType')
         default_encryption_type.assertEqual(config['defaults']['encryption'])
 
+    @udiskstestcase.tag_test(udiskstestcase.TestTags.UNSTABLE)
+    def test_integrity(self):
+        passwd = 'test'
+
+        cryptsetup_version = _get_cryptsetup_version()
+        if cryptsetup_version < LooseVersion('2.2.0'):
+            self.skipTest('Integrity devices are not marked as internal in cryptsetup < 2.2.0')
+
+        device = self.get_device(self.vdevs[0])
+        self._create_luks_integrity(self.vdevs[0], passwd)
+
+        self.addCleanup(self._remove_luks, device)
+        self.udev_settle()
+
+        self.assertHasIface(device, self.iface_prefix + '.Encrypted')
+
+        # the device is not opened, we need to read the UUID from LUKS metadata
+        luks_uuid = BlockDev.crypto_luks_uuid(self.vdevs[0])
+
+        luks_path = device.Unlock('test', self.no_options,
+                                  dbus_interface=self.iface_prefix + '.Encrypted')
+        self.assertIsNotNone(luks_path)
+        self.assertTrue(os.path.exists('/dev/disk/by-uuid/%s' % luks_uuid))
+
+        luks = self.bus.get_object(self.iface_prefix, luks_path)
+        self.assertIsNotNone(luks)
+
+        crypto_dev = self.get_property(luks, '.Block', 'CryptoBackingDevice')
+        crypto_dev.assertEqual(device.object_path)
+
+        dbus_cleartext = self.get_property(device, '.Encrypted', 'CleartextDevice')
+        dbus_cleartext.assertEqual(luks_path)
+
+        device.Lock(self.no_options, dbus_interface=self.iface_prefix + '.Encrypted')
+
+        dbus_cleartext = self.get_property(device, '.Encrypted', 'CleartextDevice')
+        dbus_cleartext.assertEqual('/')
+
+
+class UdisksEncryptedTestBITLK(udiskstestcase.UdisksTestCase):
+
+    # we can't create BitLocker formats using libblockdev
+    # so we are using these images from cryptsetup test suite
+    # https://gitlab.com/cryptsetup/cryptsetup/blob/master/tests/bitlk-images.tar.xz
+    bitlk_img = "bitlk-aes-xts-128.img"
+    passphrase = "anaconda"
+    tempdir = None
+
+    def tearDown(self):
+        try:
+            self.loop.Lock(self.no_options, dbus_interface=self.iface_prefix + '.Encrypted')
+        except dbus.exceptions.DBusException as e:
+            # ignore when the device is actually already locked
+            if not str(e).endswith('is not unlocked'):
+                raise e
+
+        self.loop.Delete(self.no_options, dbus_interface=self.iface_prefix + '.Loop')
+        shutil.rmtree(self.tempdir)
+
+        super(UdisksEncryptedTestBITLK, self).tearDown()
+
+    def setUp(self):
+        cryptsetup_version = _get_cryptsetup_version()
+        if cryptsetup_version < LooseVersion('2.3.0'):
+            self.skipTest('BITLK not supported by cryptsetup')
+
+        blkid_version = _get_blkid_version()
+        if blkid_version < LooseVersion('2.33'):
+            self.skipTest('BITLK not supported by blkid')
+
+        self.manager = self.get_interface('/Manager', '.Manager')
+        self.tempdir = tempfile.mkdtemp(prefix='udisks_test_bitlk')
+        images = os.path.join(os.path.dirname(__file__), 'bitlk-images.tar.gz')
+        with tarfile.open(images, 'r') as tar:
+            tar.extractall(self.tempdir)
+
+        with open(os.path.join(self.tempdir, self.bitlk_img), 'r+b') as loop_file:
+            fd = loop_file.fileno()
+            loop_path = self.manager.LoopSetup(fd, self.no_options)
+            self.assertIsNotNone(loop_path)
+            self.loop = self.bus.get_object(self.iface_prefix, loop_path)
+
+        super(UdisksEncryptedTestBITLK, self).setUp()
+
+    def test_open_close(self):
+        self.assertHasIface(self.loop, self.iface_prefix + '.Encrypted')
+
+        crypt_path = self.loop.Unlock(self.passphrase, self.no_options,
+                                      dbus_interface=self.iface_prefix + '.Encrypted')
+        self.assertIsNotNone(crypt_path)
+        crypt_dev = self.bus.get_object(self.iface_prefix, crypt_path)
+        self.assertIsNotNone(crypt_dev)
+
+        dbus_cleartext = self.get_property(self.loop, '.Encrypted', 'CleartextDevice')
+        dbus_cleartext.assertEqual(str(crypt_path))
+        dbus_type = self.get_property(self.loop, '.Encrypted', 'HintEncryptionType')
+        dbus_type.assertEqual("BITLK")
+
+        dbus_backing = self.get_property(crypt_dev, '.Block', 'CryptoBackingDevice')
+        dbus_backing.assertEqual(self.loop.object_path)
+        dbus_fs = self.get_property(crypt_dev, '.Block', 'IdType')
+        dbus_fs.assertEqual("ntfs")
+
+        self.loop.Lock(self.no_options, dbus_interface=self.iface_prefix + '.Encrypted')
+
+        dbus_cleartext = self.get_property(self.loop, '.Encrypted', 'CleartextDevice')
+        dbus_cleartext.assertEqual('/')
+
+        # check that the DM crypt device disappears after lock
+        udisks = self.get_object('')
+        objects = udisks.GetManagedObjects(dbus_interface='org.freedesktop.DBus.ObjectManager')
+        self.assertNotIn(str(crypt_path), objects.keys())
+
+
 del UdisksEncryptedTest  # skip UdisksEncryptedTest
