@@ -68,7 +68,8 @@ enum
 };
 
 G_DEFINE_ABSTRACT_TYPE_WITH_CODE (UDisksBaseJob, udisks_base_job, UDISKS_TYPE_JOB_SKELETON,
-                                  G_IMPLEMENT_INTERFACE (UDISKS_TYPE_JOB, job_iface_init));
+                                  G_IMPLEMENT_INTERFACE (UDISKS_TYPE_JOB, job_iface_init)
+                                  G_ADD_PRIVATE (UDisksBaseJob));
 
 static void
 udisks_base_job_finalize (GObject *object)
@@ -168,7 +169,7 @@ udisks_base_job_init (UDisksBaseJob *job)
 {
   gint64 now_usec;
 
-  job->priv = G_TYPE_INSTANCE_GET_PRIVATE (job, UDISKS_TYPE_BASE_JOB, UDisksBaseJobPrivate);
+  job->priv = udisks_base_job_get_instance_private (job);
 
   now_usec = g_get_real_time ();
   udisks_job_set_start_time (UDISKS_JOB (job), now_usec);
@@ -233,8 +234,6 @@ udisks_base_job_class_init (UDisksBaseJobClass *klass)
                                                          G_PARAM_READABLE |
                                                          G_PARAM_WRITABLE |
                                                          G_PARAM_STATIC_STRINGS));
-
-  g_type_class_add_private (klass, sizeof (UDisksBaseJobPrivate));
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -368,7 +367,6 @@ handle_cancel (UDisksJob              *_job,
   const gchar *action_id;
   const gchar *message;
   uid_t caller_uid;
-  gid_t caller_gid;
   GError *error = NULL;
 
   object = udisks_daemon_util_dup_object (job, &error);
@@ -382,8 +380,6 @@ handle_cancel (UDisksJob              *_job,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &caller_gid,
-                                               NULL,
                                                &error))
     {
       g_dbus_method_invocation_take_error (invocation, error);
