@@ -187,6 +187,7 @@ udisks_linux_loop_update (UDisksLinuxLoop        *loop,
     }
   udisks_loop_set_setup_by_uid (UDISKS_LOOP (loop), setup_by_uid);
 
+  g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (loop));
   g_object_unref (device);
 }
 
@@ -201,7 +202,7 @@ handle_delete (UDisksLoop            *loop,
   UDisksObject *object = NULL;
   UDisksBlock *block;
   UDisksDaemon *daemon = NULL;
-  UDisksState *state;
+  UDisksState *state = NULL;
   GError *error = NULL;
   uid_t caller_uid;
   uid_t setup_by_uid;
@@ -219,6 +220,9 @@ handle_delete (UDisksLoop            *loop,
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
   state = udisks_daemon_get_state (daemon);
 
+  udisks_linux_block_object_lock_for_cleanup (UDISKS_LINUX_BLOCK_OBJECT (object));
+  udisks_state_check_block (state, udisks_linux_block_object_get_device_number (UDISKS_LINUX_BLOCK_OBJECT (object)));
+
   error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon, invocation, NULL, &caller_uid, &error))
     {
@@ -284,6 +288,10 @@ handle_delete (UDisksLoop            *loop,
   udisks_loop_complete_delete (loop, invocation);
 
  out:
+  if (object != NULL)
+    udisks_linux_block_object_release_cleanup_lock (UDISKS_LINUX_BLOCK_OBJECT (object));
+  if (state != NULL)
+    udisks_state_check (state);
   g_free (device_file);
   g_clear_object (&object);
 
@@ -355,9 +363,8 @@ handle_set_autoclear (UDisksLoop             *loop,
   g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (loop));
 
   /* ... but make sure we update the property value from sysfs */
-  udisks_linux_block_object_trigger_uevent (UDISKS_LINUX_BLOCK_OBJECT (object));
-
-  /* TODO: would be better to have something like trigger_uevent_and_wait_for_it_sync() */
+  udisks_linux_block_object_trigger_uevent_sync (UDISKS_LINUX_BLOCK_OBJECT (object),
+                                                 UDISKS_DEFAULT_WAIT_TIMEOUT);
 
   udisks_loop_complete_set_autoclear (loop, invocation);
 
