@@ -744,9 +744,10 @@ handle_mdraid_create (UDisksManager         *_object,
   if (!bd_md_create (array_name, arg_level, disks, 0, NULL, FALSE, arg_chunk, NULL, &error))
     {
       g_prefix_error (&error, "Error creating RAID array: ");
-      g_dbus_method_invocation_take_error (invocation, error);
       udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
+      g_dbus_method_invocation_take_error (invocation, error);
       success = FALSE;
+      job = NULL;
       goto out;
     }
 
@@ -763,10 +764,13 @@ handle_mdraid_create (UDisksManager         *_object,
         }
       raid_device_file = g_strdup_printf ("/dev/%s", raid_node);
     }
-
   else
     raid_device_file = g_strdup (array_name);
 
+  /* trigger uevent on the newly created md node */
+  udisks_daemon_util_trigger_uevent_sync (manager->daemon, raid_device_file, NULL,
+                                          UDISKS_DEFAULT_WAIT_TIMEOUT);
+
   /* ... then, sit and wait for raid array object to show up */
   array_object = udisks_daemon_wait_for_object_sync (manager->daemon,
                                                      wait_for_array_object,
@@ -956,6 +960,15 @@ handle_enable_module (UDisksManager         *object,
   UDisksLinuxManager *manager = UDISKS_LINUX_MANAGER (object);
   EnableModulesData *data;
 
+  if (! udisks_module_validate_name (arg_name))
+    {
+      g_dbus_method_invocation_return_error (invocation,
+                                             G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
+                                             "Requested module name '%s' is not a valid udisks2 module name.",
+                                             arg_name);
+      return TRUE;
+    }
+
   if (! arg_enable)
     {
       /* TODO: implement proper module unloading */
