@@ -43,6 +43,7 @@
 #include "udiskslinuxdevice.h"
 #include "udiskslinuxblock.h"
 #include "udiskssimplejob.h"
+#include "udisksstate.h"
 
 /**
  * SECTION:udiskslinuxpartition
@@ -206,8 +207,7 @@ static void update_partitions_list (UDisksObject           *disk_object,
   udisks_partition_table_set_partitions (table, (const gchar**) partitions);
 
 out:
-  if (partitions)
-    g_strfreev (partitions);
+  g_strfreev (partitions);
 }
 
 /**
@@ -243,7 +243,7 @@ udisks_linux_partition_update (UDisksLinuxPartition   *partition,
       type = g_udev_device_get_property (device->udev_device, "ID_PART_ENTRY_TYPE");
       offset = g_udev_device_get_property_as_uint64 (device->udev_device, "ID_PART_ENTRY_OFFSET") * G_GUINT64_CONSTANT (512);
       size = g_udev_device_get_property_as_uint64 (device->udev_device, "ID_PART_ENTRY_SIZE") * G_GUINT64_CONSTANT (512);
-      name = udisks_decode_udev_string (g_udev_device_get_property (device->udev_device, "ID_PART_ENTRY_NAME"));
+      name = udisks_decode_udev_string (g_udev_device_get_property (device->udev_device, "ID_PART_ENTRY_NAME"), NULL);
       uuid = g_udev_device_get_property (device->udev_device, "ID_PART_ENTRY_UUID");
       flags = g_udev_device_get_property_as_uint64 (device->udev_device, "ID_PART_ENTRY_FLAGS");
 
@@ -312,6 +312,8 @@ udisks_linux_partition_update (UDisksLinuxPartition   *partition,
   udisks_partition_set_is_container (UDISKS_PARTITION (partition), is_container);
   udisks_partition_set_is_contained (UDISKS_PARTITION (partition), is_contained);
 
+  g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (partition));
+
   g_free (name);
   g_clear_object (&device);
   g_clear_object (&disk_block_object);
@@ -328,6 +330,7 @@ handle_set_flags (UDisksPartition       *partition,
   UDisksBlock *block = NULL;
   UDisksObject *object = NULL;
   UDisksDaemon *daemon = NULL;
+  UDisksState *state = NULL;
   gchar *device_name = NULL;
   UDisksObject *partition_table_object = NULL;
   UDisksPartitionTable *partition_table = NULL;
@@ -353,6 +356,7 @@ handle_set_flags (UDisksPartition       *partition,
     }
 
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
+  state = udisks_daemon_get_state (daemon);
   block = udisks_object_get_block (object);
   partition_table_object = udisks_daemon_find_object (daemon, udisks_partition_get_table (partition));
   partition_table = udisks_object_get_partition_table (partition_table_object);
@@ -360,6 +364,9 @@ handle_set_flags (UDisksPartition       *partition,
   device_name = udisks_block_dup_device (partition_table_block);
   partition_name = udisks_block_dup_device (block);
 
+  udisks_linux_block_object_lock_for_cleanup (UDISKS_LINUX_BLOCK_OBJECT (object));
+  udisks_state_check_block (state, udisks_linux_block_object_get_device_number (UDISKS_LINUX_BLOCK_OBJECT (object)));
+
   /* hold a file descriptor open to suppress BLKRRPART generated by the tools */
   fd = open (partition_name, O_RDONLY);
 
@@ -436,13 +443,18 @@ handle_set_flags (UDisksPartition       *partition,
         goto out;
       }
 
-  udisks_linux_block_object_trigger_uevent (UDISKS_LINUX_BLOCK_OBJECT (object));
+  udisks_linux_block_object_trigger_uevent_sync (UDISKS_LINUX_BLOCK_OBJECT (object),
+                                                 UDISKS_DEFAULT_WAIT_TIMEOUT);
   udisks_partition_complete_set_flags (partition, invocation);
   udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
 
  out:
   if (fd != -1)
     close (fd);
+  if (object != NULL)
+    udisks_linux_block_object_release_cleanup_lock (UDISKS_LINUX_BLOCK_OBJECT (object));
+  if (state != NULL)
+    udisks_state_check (state);
   g_free (device_name);
   g_free (partition_name);
   g_clear_error (&error);
@@ -468,6 +480,7 @@ handle_set_name (UDisksPartition       *partition,
   UDisksBlock *block = NULL;
   UDisksObject *object = NULL;
   UDisksDaemon *daemon = NULL;
+  UDisksState *state = NULL;
   gchar *device_name = NULL;
   gchar *partition_name = NULL;
   UDisksObject *partition_table_object = NULL;
@@ -491,8 +504,12 @@ handle_set_name (UDisksPartition       *partition,
     }
 
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
+  state = udisks_daemon_get_state (daemon);
   block = udisks_object_get_block (object);
 
+  udisks_linux_block_object_lock_for_cleanup (UDISKS_LINUX_BLOCK_OBJECT (object));
+  udisks_state_check_block (state, udisks_linux_block_object_get_device_number (UDISKS_LINUX_BLOCK_OBJECT (object)));
+
   partition_table_object = udisks_daemon_find_object (daemon, udisks_partition_get_table (partition));
   partition_table = udisks_object_get_partition_table (partition_table_object);
   partition_table_block = udisks_object_get_block (partition_table_object);
@@ -553,13 +570,18 @@ handle_set_name (UDisksPartition       *partition,
       goto out;
     }
 
-  udisks_linux_block_object_trigger_uevent (UDISKS_LINUX_BLOCK_OBJECT (object));
+  udisks_linux_block_object_trigger_uevent_sync (UDISKS_LINUX_BLOCK_OBJECT (object),
+                                                 UDISKS_DEFAULT_WAIT_TIMEOUT);
   udisks_partition_complete_set_name (partition, invocation);
   udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
 
  out:
   if (fd != -1)
     close (fd);
+  if (object != NULL)
+    udisks_linux_block_object_release_cleanup_lock (UDISKS_LINUX_BLOCK_OBJECT (object));
+  if (state != NULL)
+    udisks_state_check (state);
   g_free (device_name);
   g_free (partition_name);
   g_clear_error (&error);
@@ -742,6 +764,9 @@ udisks_linux_partition_set_type_sync (UDisksLinuxPartition  *partition,
       goto out;
     }
 
+  udisks_linux_block_object_trigger_uevent_sync (UDISKS_LINUX_BLOCK_OBJECT (object),
+                                                 UDISKS_DEFAULT_WAIT_TIMEOUT);
+
   ret = TRUE;
   udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
 
@@ -829,6 +854,7 @@ handle_resize (UDisksPartition       *partition,
   UDisksBlock *block = NULL;
   UDisksObject *object = NULL;
   UDisksDaemon *daemon = NULL;
+  UDisksState *state = NULL;
   UDisksObject *partition_table_object = NULL;
   UDisksBlock *partition_table_block = NULL;
   uid_t caller_uid;
@@ -854,11 +880,15 @@ handle_resize (UDisksPartition       *partition,
   wait_data.object_path = g_dbus_object_get_object_path (G_DBUS_OBJECT (object));
   wait_data.new_size = 0; /* hit timeout in case of error */
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
+  state = udisks_daemon_get_state (daemon);
   block = udisks_object_get_block (object);
   part = udisks_block_get_device (block);
   partition_table_object = udisks_daemon_find_object (daemon, udisks_partition_get_table (partition));
   partition_table_block = udisks_object_get_block (partition_table_object);
 
+  udisks_linux_block_object_lock_for_cleanup (UDISKS_LINUX_BLOCK_OBJECT (object));
+  udisks_state_check_block (state, udisks_linux_block_object_get_device_number (UDISKS_LINUX_BLOCK_OBJECT (object)));
+
   job = udisks_daemon_launch_simple_job (daemon,
                                          UDISKS_OBJECT (object),
                                          "partition-modify",
@@ -905,17 +935,23 @@ handle_resize (UDisksPartition       *partition,
     udisks_warning ("Could not open %s to query new partition size", part);
   }
 
+  udisks_linux_block_object_trigger_uevent_sync (UDISKS_LINUX_BLOCK_OBJECT (partition_table_object ? partition_table_object : object),
+                                                 UDISKS_DEFAULT_WAIT_TIMEOUT);
   partition_object = udisks_daemon_wait_for_object_sync (daemon,
                                                          wait_for_partition_resize,
                                                          &wait_data,
                                                          NULL,
-                                                         10,
+                                                         UDISKS_DEFAULT_WAIT_TIMEOUT,
                                                          NULL);
 
   udisks_partition_complete_resize (partition, invocation);
   udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
 
  out:
+  if (object != NULL)
+    udisks_linux_block_object_release_cleanup_lock (UDISKS_LINUX_BLOCK_OBJECT (object));
+  if (state != NULL)
+    udisks_state_check (state);
   g_clear_error (&error);
   g_clear_object (&object);
   g_clear_object (&block);
@@ -938,6 +974,7 @@ handle_delete (UDisksPartition       *partition,
   UDisksBlock *block = NULL;
   UDisksObject *object = NULL;
   UDisksDaemon *daemon = NULL;
+  UDisksState *state = NULL;
   gchar *device_name = NULL;
   gchar *partition_name = NULL;
   UDisksObject *partition_table_object = NULL;
@@ -962,10 +999,14 @@ handle_delete (UDisksPartition       *partition,
     }
 
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
+  state = udisks_daemon_get_state (daemon);
   block = udisks_object_get_block (object);
   partition_table_object = udisks_daemon_find_object (daemon, udisks_partition_get_table (partition));
   partition_table_block = udisks_object_get_block (partition_table_object);
 
+  udisks_linux_block_object_lock_for_cleanup (UDISKS_LINUX_BLOCK_OBJECT (object));
+  udisks_state_check_block (state, udisks_linux_block_object_get_device_number (UDISKS_LINUX_BLOCK_OBJECT (object)));
+
   if (teardown_flag)
     {
       if (!udisks_linux_block_teardown (block, invocation, options, &error))
@@ -1006,12 +1047,17 @@ handle_delete (UDisksPartition       *partition,
       goto out;
     }
   /* this is sometimes needed because parted(8) does not generate the uevent itself */
-  udisks_linux_block_object_trigger_uevent (UDISKS_LINUX_BLOCK_OBJECT (partition_table_object));
+  udisks_linux_block_object_trigger_uevent_sync (UDISKS_LINUX_BLOCK_OBJECT (partition_table_object),
+                                                 UDISKS_DEFAULT_WAIT_TIMEOUT);
 
   udisks_partition_complete_delete (partition, invocation);
   udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
 
  out:
+  if (object != NULL)
+    udisks_linux_block_object_release_cleanup_lock (UDISKS_LINUX_BLOCK_OBJECT (object));
+  if (state != NULL)
+    udisks_state_check (state);
   g_free (device_name);
   g_free (partition_name);
   g_clear_error (&error);
